% *************************************************************************
% **************************** The OEOA Toolbox ***************************
% The On-line Ensemble Using Ordered Aggregation (OEOA) Toolbox
% *************************************************************************
% An Adaptive Ensemble of On-line Extreme Learning Machines with Variable 
% Forgetting Factor for Dynamic System Prediction
% *************************************************************************
% Version 1.0 Nov/27/2015
%
% Developed in MATLAB R2010a
% 
% Author and programmer: Symone Gomes Soares
%
% e-mail: symonesoares@gmail.com 
%
% Homepage: http://sites.google.com/site/symonesoares/
%   
% Paper: SOARES, Symone Gomes; ARAÚJO, Rui. An Adaptive Ensemble of On-line 
% Extreme Learning Machines with Variable Forgetting Factor for Dynamic 
% System Prediction. Neurocomputing, v. 171, p. 693-707, 2016. 
%        DOI: http://dx.doi.org/10.1016/j.neucom.2015.07.035
%
% *************************************************************************
% *************************************************************************
% An Example of the OEOA algorithm using the *On-line Sequential Extreme
% Learning Machine with Directional Forgetting Factor (DFF-OS-ELM) and the 
% **FCCU data set.
%
%  *SOARES, Symone Gomes; ARAÚJO, Rui. An Adaptive Ensemble of On-line 
%   Extreme Learning Machines with Variable Forgetting Factor for Dynamic 
%   System Prediction. Neurocomputing, v. 171, p. 693-707, 2016. 
%
%
% **LIU, Y.; Hu, N.; WANG, H.; LI, P. 2009. Soft Chemical Analyzer 
%   Development Using Adaptive Least-Squares Support Vector Regression With 
%   Selective Pruning and Variable Moving Window Size. Ind. Eng. Chem. Res. 
%   48 (May (12)), 5731–5741.
%
% *************************************************************************

clear; clc; close all;

% Loading the FCCU data set
% Using only the first output (gasoline concentration)
D      = load('FCCU.txt'); 
input  = D(:,1:6); % attributing the inputs  
output = D(:,7);   % attributing the output  

% Parameter Setting
fprintf('*** Testing The OEOA Toolbox***\n');
fprintf('    Example of the OEOA algorithm using the DFF-OS-ELM and the FCCU data set\n\n');
kfold   = 10;    % number of folds for the k-fold cross-validation
sW      = 10;    % window's size (sW >= kfold)
maxNHL  = 20;    % maximum number of hidden nodes in OS-ELM (maxHL >= sW)
Alpha   = 0.04;  % factor to include a new model (0 < Alpha < 1);     
ActF    = 'sig'; % activation function for OS-ELM   


% Calling fOEOA_OSELM with no ordered aggregation
M_max   = 15;    % maximum number of models in the ensemble (M_max > 1) and 
                 % (M_max >= M_min)
M_min   = 15;    % minimum number of models in the ensemble (M_min > 1) and 
                 % (M_max >= M_min)
fprintf('*** Starting fOEOA_DFF_OSELM function with no Ordered Aggregation (OA)***');
[CC, MSE, RMSE, YR, YP, time] = fOEOA_DFF_OSELM(input, output, sW, maxNHL, Alpha, kfold, ActF, M_max, M_min);


% Calling fOEOA_OSELM with ordered aggregation
M_max   = 30;    % maximum number of models in the ensemble (M_max > 1) and 
                 % (M_max >= M_min)
M_min   = 10;    % minimum number of models in the ensemble (M_min > 1) and 
                 % (M_max >= M_min)
fprintf('\n*** Starting fOEOA_DFF_OSELM function with Ordered Aggregation (OA)***');
[CC2, MSE2, RMSE2, YR2, YP2, time2] = fOEOA_DFF_OSELM(input, output, sW, maxNHL, Alpha, kfold, ActF, M_max, M_min);


% Plotting Results
fprintf('\n*** Ploting Testing Data ***\n');
figure;
plot(YR, 'k');
hold on;
plot(YP, 'r');
hold on;
plot(YP2, 'b');
xlim([1 size(YR,2)]);
legend('Real Output', 'Predicted Output (no OA)', 'Predicted Output (OA)', 2);
