% *************************************************************************
% **************************** The OEOA Toolbox ***************************
% The On-line Ensemble Using Ordered Aggregation (OEOA) Toolbox
% *************************************************************************
% An Adaptive Ensemble of On-line Extreme Learning Machines with Variable 
% Forgetting Factor for Dynamic System Prediction
% *************************************************************************
% Version 1.0 Nov/27/2015
%
% Developed in MATLAB R2010a
% 
% Author and programmer: Symone Gomes Soares
%
% e-mail: symonesoares@gmail.com 
%
% Homepage: http://sites.google.com/site/symonesoares/
%   
% Paper: SOARES, Symone Gomes; ARAÚJO, Rui. An Adaptive Ensemble of On-line 
% Extreme Learning Machines with Variable Forgetting Factor for Dynamic 
% System Prediction. Neurocomputing, v. 171, p. 693-707, 2016. 
%        DOI: http://dx.doi.org/10.1016/j.neucom.2015.07.035
%
% *************************************************************************
% *************************************************************************
% *************************************************************************
% Algorithm Description:
%    The retraining of the OS-ELM algorithm for regression 
% *************************************************************************
% Inputs of the Algorithm:
%
% - ELM: parameters of the OS-ELM model
%
% - xi(1 x number of input variables) : input vector
% 
% - yi(output variable): output point
%   ps.: it accepts only one output variable
%
% *************************************************************************
% Outputs of the Algorithm:
%
% - ELM: new parameters of the OS-ELM model
%
% - yiP: predicted output after retraining the OS-ELM model
%
% *************************************************************************
% *************************************************************************
% *************************************************************************

function [ELM, yiP] = OSELM_Reg_Retrain(ELM, xi, yi)

    nS = size(xi,1);

    %%%%%%%%%%%%% step 2 Sequential Learning Phase
    switch lower(ELM.ActivationFunction)
        case{'rbf'}
            H = RBFun(xi,ELM.IW,ELM.Bias);
        case{'sig'}
            H = SigActFun(xi,ELM.IW,ELM.Bias);
        case{'sin'}
            H = SinActFun(xi,ELM.IW,ELM.Bias);
        case{'hardlim'}
            H = HardlimActFun(xi,ELM.IW,ELM.Bias);
    end    
    ELM.M = ELM.M - ELM.M * H' * (eye(nS) + H * ELM.M * H')^(-1) * H * ELM.M; 
    ELM.beta = ELM.beta + ELM.M * H' * (yi - H * ELM.beta);

    yiP = H * ELM.beta;
    clear HTrain;

end
 
