% *************************************************************************
% **************************** The OEOA Toolbox ***************************
% The On-line Ensemble Using Ordered Aggregation (OEOA) Toolbox
% *************************************************************************
% An Adaptive Ensemble of On-line Extreme Learning Machines with Variable 
% Forgetting Factor for Dynamic System Prediction
% *************************************************************************
% Version 1.0 Nov/27/2015
%
% Developed in MATLAB R2010a
% 
% Author and programmer: Symone Gomes Soares
%
% e-mail: symonesoares@gmail.com 
%
% Homepage: http://sites.google.com/site/symonesoares/
%   
% Paper: SOARES, Symone Gomes; ARAÚJO, Rui. An Adaptive Ensemble of On-line 
% Extreme Learning Machines with Variable Forgetting Factor for Dynamic 
% System Prediction. Neurocomputing, v. 171, p. 693-707, 2016. 
%        DOI: http://dx.doi.org/10.1016/j.neucom.2015.07.035
%
% *************************************************************************
% *************************************************************************
% *************************************************************************
% Algorithm Description:
%    The testing of the OS-ELM algorithm for regression 
% *************************************************************************
% Inputs of the Algorithm:
%
% - Xtest(number of samples x number of input variables) : input matrix or
%   input vector
% 
% - Ytest(number of samples x output variable): output vector or output 
%   point
%   ps.: it accepts only one output variable
%
% - ELM: parameters of the OS-ELM model
% 
% *************************************************************************
% Outputs of the Algorithm:
%
% - TestAcc: accuracy of the model in the testing; it includes correlation 
%            coefficient, mean square error, and root mean square error
%
% - TestOutput: testing output(s) of the OS-ELM model
%
% *************************************************************************
% *************************************************************************
% *************************************************************************

function [TestAcc, TestOutput] = OSELM_Reg_Test(Xtest, Ytest, ELM)

    %%%%%%%%%%% Performance Evaluation
    switch lower(ELM.ActivationFunction)
        case{'rbf'}
            HTest = RBFun(Xtest, ELM.IW, ELM.Bias);
        case{'sig'}
            HTest = SigActFun(Xtest, ELM.IW, ELM.Bias);
        case{'sin'}
            HTest = SinActFun(Xtest, ELM.IW, ELM.Bias);
        case{'hardlim'}
            HTest = HardlimActFun(Xtest, ELM.IW, ELM.Bias);
    end    
    TestOutput = HTest * ELM.beta;
    clear HTest;

    TestAcc.rmse=sqrt(mse(Ytest - TestOutput));     %   Calculate testing accuracy (RMSE) for regression case
    TestAcc.mse=mse(Ytest - TestOutput);            %   Calculate testing accuracy (MSE) for regression case
    TestAcc.cc=corr2(Ytest, TestOutput);            %   Calculate testing accuracy (CC) for regression case

end

